#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "fn.h"
#define NUMSP 80
/*= bootstrap data sets */
void boot_seq(int numsp, int nsite, char *oseq, char *seq){
  int i,j,k;
  double u;
  for(i = 0; i < nsite; i++){
    u = drand48();
    for(j = 1; j <= nsite; j++)
      if (u <= ((double) j)/((double) nsite)) break;
    for(k = 0; k < numsp; k++) seq[i+k*nsite] = oseq[j-1+k*nsite];
  }
}

/* read in data, original + B bootstrapped samples */
void sboot_seq_file(FILE *infile, FILE *outfile, int B){
  int b, numsp, nsite;
  char name[NUMSP][11],*seqcb,*seqc;

  fscanf(infile, "%i %i", &numsp, &nsite);
  seqc = (char *) malloc((size_t) numsp*nsite*sizeof(char));
  rinterleavef_nolim(infile,&numsp,&nsite,name,seqc);
  winterleavef(outfile,numsp,nsite,name,seqc); /* first block is original data */

  seqcb = (char *) malloc((size_t) (B+1)*numsp*nsite*sizeof(char));
  for(b = 0; b < B; b++){
    boot_seq(numsp,nsite,seqc,&seqcb[b*numsp*nsite]);
    winterleavef(outfile,numsp,nsite,name,&seqcb[b*numsp*nsite]);
  }
  free(seqcb);
  free(seqc);
}

/* read in data, original + B bootstrapped samples + B double bootstrap samples 
 * for each original bootstrap sample */
void dboot_seq_file(FILE *infile, FILE *outfile, int B){
  int b,db,numsp, nsite;
  char name[NUMSP][11],*seqcb,*seqc;

  fscanf(infile, "%i %i", &numsp, &nsite);
  seqc = (char *) malloc((size_t) numsp*nsite*sizeof(char));
  rinterleavef_nolim(infile,&numsp,&nsite,name,seqc);
  winterleavef(outfile,numsp,nsite,name,seqc); /* first block is original data */

  seqcb = (char *) malloc((size_t) (B+1)*numsp*nsite*sizeof(char));
  for(b = 0; b < B; b++){
    boot_seq(numsp,nsite,seqc,&seqcb[b*numsp*nsite]);
    winterleavef(outfile,numsp,nsite,name,&seqcb[b*numsp*nsite]);
  }
  for(b = 0; b < B; b++)
    for(db = 0; db < B; db++){
      boot_seq(numsp,nsite,&seqcb[b*numsp*nsite],
	       &seqcb[B*numsp*nsite]);
      winterleavef(outfile,numsp,nsite,name,&seqcb[B*numsp*nsite]);
    }
  free(seqcb);
  free(seqc);
}
