#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "tree2splitf.h"

/*= convert split to gp split; 0 if w/in gp split or terminal */
int spl2gspl(int ngps, int *split, int *gpidx, int *gsplit){
  int num_ones=0,gp;
  /* first member of gp gives side of split
   * if w/in gp split, number of 1's or 0's <= 1 */
  for(gp = 0; gp < ngps; gp++)
    gsplit[gp] = split[gpidx[gp]]; 
  for(gp = 0; gp < ngps; gp++) num_ones += gsplit[gp];
  if (num_ones <= 1 || ngps-num_ones <= 1) 
    return(0);
  else{
    split_leading1(ngps,gsplit);
    return(1);
  }
}

/*= convert splits to gsplits; return 1 if compatible */
int conv2gpsplits(int numsp, int ngps, int *splits, int *gpidx, int *gpsplits, 
		  int *bp_splits, int *gsplits){
  int gspl=0,gp,spl;

  for(gp = 0; gp < ngps; gp++)
    for(spl = 0; spl < numsp-3; spl++) /* check compatible */
      if (!compat_pair(numsp,&splits[spl*numsp],&gpsplits[gp*numsp])) return(0);  
  for(spl = 0; spl < numsp-3; spl++) /* convert 2 gpsplits */
    if (spl2gspl(ngps,&splits[spl*numsp],gpidx,&gsplits[gspl*ngps])){
      bp_splits[gspl] = bp_splits[spl];
      gspl++;
    }
  return(1);
}

/*= create gpsplits, gpnames, gpidx */
int cvsame(int n, char *x, char *y){
  int i;
  for(i = 0; i < n; i++)
    if (x[i] != y[i]) return(0);
  return(1);
}

int gidx_name(int numsp, char name[][11], char *cname){
  int i,j,taxa;
  for(i = 0; i < 10; i++) /* pad cname w/ blanks */
    if (cname[i] == '\0'){
      for(j = i; j < 10; j++) cname[j] = ' ';
      break;
    }
  cname[10] = '\0';
  for(taxa = 0; taxa < numsp; taxa++)
    if (cvsame(10, name[taxa], cname)) return(taxa);
  return(-1);
}

int gpspl_mk(FILE *gpfile, int numsp, char name[][11], char gpname[][11], 
	     int *gpidx, int *gpsplits){
  int ngps,gp,nwgp,i,gpm,idx;
  char cname[11];

  fscanf(gpfile,"%i",&ngps);
  for(gp = 0; gp < ngps; gp++){
    fscanf(gpfile,"%s",gpname[gp]);
    fscanf(gpfile,"%i",&nwgp);
    for(i = 0; i < numsp; i++) gpsplits[i+gp*numsp] = 0;
    for(gpm = 0; gpm < nwgp; gpm++){
      fscanf(gpfile,"%s",cname);
      idx = gidx_name(numsp,name,cname);
      if (idx < 0){
	fprintf(stderr,"Names of taxa in Group %i do not match data names\n",gpm+1);
	exit(1);
      }
      if (gpm == 0) gpidx[gp] = idx;
      gpsplits[idx+gp*numsp] = 1;
    }
    split_leading1(numsp,&gpsplits[gp*numsp]);
  }
  return(ngps);
}
