/* Obtain ntree trees that have splits in the trees obtained from bootstrap
 * samples, ranked according to their minbp
 *
 * ranked_spl ntree B treefile < infile
 *
 * treefile: should contain NEWICK format trees
 * estimated for the following data sets:
 * 
 *   original data
 *   b1: bootstrap data set 1 of original data
 *           ....
 *   bB: bootstrap data set B of original data
 *
 * outfiles:
 *
 * minbp.newick: The ntree trees, ordered from the tree with largest minBP to
 * smallest. If the number of trees with that have splits in the obtained from
 * bootstrap samples is smaller than ntree, all of the those trees ar output
 *
 * minbp.minbp: The minBP values for the trees in minbp.newick
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "fn.h"
#include "tree2splitf.h"

int main(int argc, char **argv)
{
#define NUMSP 60
  FILE *newick_outfile,*treefile,*outfile;
  int tr,ntree,numsp,nsite,B,nspl,maxtrees,ntrees,*count,*trees;
  int *minbpv;
  int *usplits,*splits;
  char name[NUMSP][11],*seqc;
  double *utreec;
  
  treefile = fopen(argv[--argc],"r");
  B = atoi(argv[--argc]);
  ntree = atoi(argv[--argc]);

  maxtrees = 1000000;
  scanf("%i %i", &numsp,&nsite);
  seqc = (char *) malloc((size_t) nsite*numsp*sizeof(char));
  rinterleavef_nolim(stdin,&numsp,&nsite,name,seqc);
  splits=(int *) malloc((size_t) numsp*(numsp-3)*sizeof(int));
  nspl=dim_split(numsp,B);
  usplits=(int *) malloc((size_t) nspl*numsp*sizeof(int));
  count=(int *) malloc((size_t) nspl*sizeof(int));
  trees = (int *) malloc((size_t) maxtrees*(numsp-3)*sizeof(int));
  utreec =(double *) malloc((size_t) (numsp-1)*4*sizeof(double));
  minbpv = (int *) malloc((size_t) maxtrees*sizeof(int));

  nspl = usplitf(treefile,B,numsp,name,splits,usplits,count);
  fclose(treefile);
  ntrees = ranked_compat_splits(B,numsp,nspl,ntree,usplits,count,trees);

  minbp_trees(numsp,ntrees,trees,count,minbpv);

  newick_outfile = fopen("minbp.newick","w");
  for(tr = 0; tr < ntrees; tr++){
    splits2tree(B,numsp,nspl,&trees[tr*(numsp-3)],usplits,count,utreec);
    pr_newick(newick_outfile, numsp, name, utreec);
  }
  fclose(newick_outfile);

  outfile = fopen("minbp.minbp","w");
  for(tr = 0; tr < ntrees; tr++) fprintf(outfile,"%i\n",minbpv[tr]);
  fclose(outfile);

  free(usplits); free(count); free(splits); free(trees); free(utreec);
  free(minbpv);
  return(0);
}
