/* change tree to collection of splits */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "fn.h"
#include "treecfn.h"
#include "ealgc.h"
void ut2n(int numsp, double *utreec, char names[][11], char *tstr);
#define NUMSP 80

void pr_splits(FILE *outfile, int numsp, int nspl, int *splits){
  int split,i;
  for(split = 0; split < nspl; split++){
    for(i = 0; i < numsp; i++) fprintf(outfile, "%2i ",splits[i+split*numsp]);
    fprintf(outfile, "\n");
  }
}

void pr_usplits(FILE *outfile, int nspl, int numsp, int *usplits, int *count){
  int i,j;
  for(i = 0; i < nspl; i++){
    printf("%4i: ", i);
    for(j = 0; j < numsp; j++) fprintf(outfile, "%i ",usplits[j+i*numsp]);
    fprintf(outfile, "%i \n",count[i]);
  }
}

/*= splits w/ a one in the initial position; for uniqueness */
void split_leading1(int numsp, int *split){
  int i;
  if (split[0] == 1) return;
  for(i = 0; i < numsp; i++) split[i] = (split[i] == 1)?0:1;
}    

void utreec2splits(int numsp, double *utreec, int *splits){
  int i,r,l,split;
  /* ith column gives split for numsp+ith branch */
  for(split = 0; split < numsp-3; split++){
    r = (int) utreec[split*4]; l = (int) utreec[1+split*4];
    if (r < numsp){
      for(i = 0; i < numsp; i++) splits[i+split*numsp] = 0;
      splits[r+split*numsp] = 1;
    }
    else
      for(i = 0; i < numsp; i++) splits[i+split*numsp] = splits[i+(r-numsp)*numsp];
    if (l < numsp)
      splits[l+split*numsp] = 1;
    else
      for(i = 0; i < numsp; i++) splits[i+split*numsp] += splits[i+(l-numsp)*numsp];
  }
  for(split = 0; split < numsp-3; split++)
    split_leading1(numsp,&splits[split*numsp]);
}

/* maximum nmumber of unique splits */
double dim_split(int numsp, int ntrees){
  /* integer 2^{numsp-1}-numsp-1 but too large numsp will convert to negative 
   * number */
  double max1=0;
/*   int i,max2=1; */
  max1 = 5*ntrees*(numsp-1);
/*   for(i = 1; i <= numsp-1; i++) max2 *= 2; */
/*   max2 -= (numsp + 1); */
/*   max1=(max1 < max2)?max1:max2; */
  return(max1);
}

/*= two vectors are the same */
int ivsame(int n, int *x, int *y){
  int i;
  for(i = 0; i < n; i++)
    if (x[i] != y[i]) return(0);
  return(1);
}

/*= update set of usplits; return index of split, no count */
int up_usplitr(int numsp, int *nspl, int *split, int *usplits){
  int ispl,i; 
 for(ispl = 0; ispl < (*nspl); ispl++)
   if (ivsame(numsp,split,&usplits[ispl*numsp])) return(ispl);
 for(i = 0; i < numsp; i++) usplits[i+(*nspl)*numsp] = split[i];
 (*nspl) += 1;
 return(((*nspl) - 1));
}

/*= update the set of usplits */
void up_usplit(int numsp, int *nspl, int *split, int *usplits, int *count){
  int i,ispl;

  for(ispl = 0; ispl < (*nspl); ispl++)
    if (ivsame(numsp,split,&usplits[ispl*numsp])){
      count[ispl] += 1;
      return;
    }
  count[(*nspl)] = 1;
  for(i = 0; i < numsp; i++) usplits[i+(*nspl)*numsp] = split[i];
  (*nspl) += 1;
}

void up_usplits(int numsp, int nspls, int *nspl, int *splits, int *usplits, 
		int *count){
  int i;
  for(i = 0; i < nspls; i++) up_usplit(numsp,nspl,&splits[i*numsp],usplits,count);
}

/*= minimum BP for a set of bootstrap samples */
int minbpf(int B, int numsp, int nspls, int nspl, int *splits, int *usplits, 
	   int *count){
  int spl,uspl,match,minbp;

  minbp = B+1;
  for(spl = 0; spl < nspls; spl++){
    match = 0;
    for(uspl = 0; uspl < nspl; uspl++){
      if (ivsame(numsp,&splits[spl*numsp],&usplits[uspl*numsp])){
	match = 1;
	minbp = (minbp > count[uspl])?count[uspl]:minbp;
	break;
      }
    }
    if (!match) return(0);
  }
  return(minbp);
}

/*= read tree from file and convert to new utreec format */
void read_tree2nutreec(FILE *treefile, int numsp, char name[][11], double *utreec){
  static char tstring[100000];
  static double outreec[4*(NUMSP-1)];

  read_tree(treefile,tstring);
  if (has_bl(tstring))
    ch_labels(numsp, tstring, name);  /* change names to integers */
  else
    ch_labels_nbl(numsp, tstring, name);  /* change names to integers */
  treecs(numsp,tstring,outreec);
  ou2nu(numsp,outreec,utreec);
  rlord_utreec(numsp,utreec);
/*   pr_utreec(stdout, numsp, utreec); */
}

/*= get the "distribution" of minbps through double bootstrap */
int minbp_dist(FILE *treefile, int B, int numsp, char name[][11], int *splits, 
		int *usplits, int *count, int *minbp){
  int b,db,nsplt,nspl,maxnspl,*countt;
  int *splitst,*usplitst;
  double *utreec;

  utreec = (double *) malloc((size_t) numsp*4*sizeof(double));
  splitst = (int *) malloc((size_t) numsp*(numsp-3)*sizeof(int));
  maxnspl = dim_split(numsp,B);
  usplitst = (int *) malloc((size_t) numsp*maxnspl*sizeof(int));
  countt = (int *) malloc((size_t) maxnspl*sizeof(int));

  read_tree2nutreec(treefile,numsp,name,utreec); /* estimated tree */
  utreec2splits(numsp,utreec,splits);

/*   printf("Estimated Tree\n"); */
/*   pr_utreec(stdout,numsp,utreec); */
/*   pr_splits(stdout,numsp,numsp-3,splits); */

  nspl=0;
  for(b = 0; b < B; b++){ /* bootstrap support for splits: usplits, count */
    read_tree2nutreec(treefile,numsp,name,utreec); 
    utreec2splits(numsp,utreec,splitst);
    up_usplits(numsp,numsp-3,&nspl,splitst,usplits,count);

/*     printf("Bootstrap Sample %i\n",b); */
/*     pr_utreec(stdout,numsp,utreec); */
/*     pr_splits(stdout,numsp,numsp-3,splitst); */
/*     printf("Updated splits\n"); */
/*     pr_usplits(stdout,nspl,numsp,usplits,count); */
  }
  for(b = 0; b < B; b++){ /* double boostrap to get distribution of min BP */
    nsplt = 0;
    for(db = 0; db < B; db++){
      read_tree2nutreec(treefile,numsp,name,utreec); 
      utreec2splits(numsp,utreec,splitst);
      up_usplits(numsp,numsp-3,&nsplt,splitst,usplitst,countt);
    }

/*     printf("Bootstrap Sample %i\n",b); */
/*     pr_splits(stdout,numsp,numsp-3,splits); printf("\n"); */
/*     pr_usplits(stdout,nsplt,numsp,usplitst,countt); */

    minbp[b] = minbpf(B,numsp,numsp-3,nsplt,splits,usplitst,countt);
  }

  free(utreec); free(splitst); free(usplitst); free(countt);
  return(nspl);
}

/*= get the unique splits and counts for a given file of trees */
int usplitf(FILE *treefile, int B, int numsp, char name[][11], int *splits, 
	    int *usplits, int *count){
  int b,nspl;
  int *splitst;
  double *utreec;

  utreec = (double *) malloc((size_t) numsp*4*sizeof(double));
  splitst = (int *) malloc((size_t) numsp*(numsp-3)*sizeof(int));

  read_tree2nutreec(treefile,numsp,name,utreec); /* estimated tree */
  utreec2splits(numsp,utreec,splits);

  nspl=0;
  for(b = 0; b < B; b++){ /* bootstrap support for splits: usplits, count */
    read_tree2nutreec(treefile,numsp,name,utreec);
    utreec2splits(numsp,utreec,splitst);
    up_usplits(numsp,numsp-3,&nspl,splitst,usplits,count);

/*     printf("Bootstrap Sample %i\n",b); */
/*     pr_utreec(stdout,numsp,utreec); */
/*     pr_splits(stdout,numsp,numsp-3,splitst); */
/*     printf("Updated splits\n"); */
/*     pr_usplits(stdout,nspl,numsp,usplits,count); */
  }
  free(utreec); free(splitst);
  return(nspl);
}

/*= convert trees in file to splits with counts */
int tree2splitf(FILE *treefile, int ntrees, int numsp, int nsite, char name[][11], 
		int *usplits, int *count){
  int tree,nspl=0;
  int *splits;  
  double *utreec;

  utreec = (double *) malloc((size_t) numsp*4*sizeof(double));
  splits=(int *) malloc((size_t) numsp*(numsp-3)*sizeof(int));

  for(tree = 0; tree < ntrees; tree++){
    read_tree2nutreec(treefile,numsp,name,utreec);
    utreec2splits(numsp,utreec,splits);
    up_usplits(numsp,numsp-3,&nspl,splits,usplits,count);
  }

  free(utreec); free(splits);
  return(nspl);
}

/*= sort splits and return those >= zeta */
int sort_splits(int numsp, int nspl, int zeta, int *usplits, int *count){
  int nsplt,i,j,*countt,*o,*uspl;

  o = (int *) malloc((size_t) nspl*sizeof(int));
  uspl = (int *) malloc((size_t) nspl*numsp*sizeof(int));
  countt = (int *) malloc((size_t) nspl*sizeof(int));

  iorder(nspl,count,o);
  for(i = 0; i < nspl; i++){
    countt[i] = count[o[i]];
    for(j = 0; j < numsp; j++) uspl[j+i*numsp] = usplits[j+o[i]*numsp];
  }
  for(i = 0; i < nspl; i++){
    count[i] = countt[nspl-1-i];
    for(j = 0; j < numsp; j++) usplits[j+i*numsp] = uspl[j+(nspl-1-i)*numsp];
  }
  nsplt = nspl;
  for(i = nspl-1; i >= 0; i--){
    if (count[i] < zeta) 
      nsplt--;
    else
      break;
  }

  free(o); free(uspl); free(countt);
  return(nsplt);
}

/*= alpha x 100% cutoff for the minbp vector */
int minbp_cut(int B, double alpha, int *minbp){
  int idx;
  isort(B,minbp);
  idx = max((int) (alpha*B) - 1, 0);
  return(minbp[idx]);
}

/*= is a pair of splits compatabile */
int compat_pair(int numsp, int *fspl, int *sspl){
  int i,ab,acb,abc,acbc;
  
  ab=0;acb=0;abc=0;acbc=0;
  for(i = 0; i < numsp; i++){
    ab += fspl[i]*sspl[i]; /* not necc. because of leading 1 */
    acb += (1-fspl[i])*sspl[i];
    abc += fspl[i]*(1-sspl[i]);
    acbc += (1-fspl[i])*(1-sspl[i]);
  }
  if (ab == 0 || acb  == 0 || abc == 0 || acbc == 0) return(1);
  return(0);
}

/*= is a split compatible with a set */
int compat_newsplit(int nspl, int set_size, int split, int *set, 
		    int *compat_matrix){
  int set_el,i;
  for(i = 0; i < set_size; i++){
    set_el = set[i];
    if (compat_matrix[set_el+split*nspl] == 0) return(0);
  }
  return(1);
}

/*= initial set of compat splits including core gp */
int init_compat_splits(int B, int nspl, int zeta, int *count, int *nsets, 
		       int *trees, int *compat_matrix){
  int set_size,spl,is_compat;

  /* usplits assumed sorted by BP */
  set_size = 0;
  /* printf("%i %i %i\n",nspl, B, zeta); */

  for(spl = 0; spl < nspl; spl++){
    /* printf("%i %i\n", count[spl], B-zeta); */
    if (count[spl] > B - zeta){
      trees[set_size] = spl;
      set_size++;
    }
    else
      break;
  }

  /* printf("set_size %i\n", set_size); */
  if (set_size > 1){ /* check compatibility of core group */
    for(spl = 1; spl < set_size; spl++){
      is_compat = compat_newsplit(nspl, spl, spl, trees, compat_matrix);
      if (!is_compat) break;
    }
    if (!is_compat) return(-1); /* splits not compat in core group */
  }

  if (set_size == 0){ /* might be no splits w/ BP >= 1-zeta */
    *nsets = nspl;
    for(spl = 0; spl < nspl; spl++) trees[spl] = spl;
    set_size = 1;
  }
  else
    *nsets = 1;
  return(set_size);
}

/*= find all trees compatible with set of splits */
int compat_splits(int B, int numsp, int nspl, int maxtrees, int zeta, int *usplits, 
		  int *count, int *trees){
  int i,j,nsets_old,set_size,set,nsets,largest_split,split,is_compat,init_set_size;
  int *compat_matrix,*trees_old;

  compat_matrix = (int *) malloc((size_t) nspl*nspl*sizeof(int));
  trees_old = (int *) malloc((size_t) maxtrees*(numsp-3)*sizeof(int));

  for(i = 0; i < nspl; i++)
    for(j = 0; j < nspl; j++)
      compat_matrix[i+j*nspl] = compat_pair(numsp,&usplits[i*numsp],
					    &usplits[j*numsp]);
/*   for(i = 0; i < nspl; i++){ */
/*     for(j = 0; j < nspl; j++) printf("%i ", compat_matrix[i+j*nspl]); */
/*     printf("\n"); */
/*   } */
  
  init_set_size = init_compat_splits(B,nspl,zeta,count,&nsets,trees,compat_matrix);
/*   printf("init_set_size: %i\n", init_set_size); */
  if (init_set_size < 0) return(0);
  for(i = 0; i < nsets*init_set_size; i++) trees_old[i] = trees[i];
  nsets_old = nsets;

  for(set_size = init_set_size; set_size < numsp-3; set_size++){
    nsets = 0;
    for(set = 0; set < nsets_old; set++){
      largest_split = trees_old[set_size-1+set*set_size];
      for(split = largest_split+1; split < nspl; split++){
	is_compat = compat_newsplit(nspl,set_size,split,&trees_old[set*set_size],
				    compat_matrix);
	if (is_compat){
	  for(i = 0; i < set_size; i++) 
	    trees[i+nsets*(set_size+1)] = trees_old[i+set*set_size];
	  trees[set_size+nsets*(set_size+1)] = split;
	  nsets++;
	}
      }
    } 
    for(i = 0; i < nsets*(set_size+1); i++) trees_old[i] = trees[i];
    nsets_old = nsets;

/*     printf("set_size nsets %i %i\n", set_size, nsets); */
/*     for(i = 0; i < nsets; i++){ */
/*       printf("%i: ", i); */
/*       for(split = 0; split < set_size+1; split++) */
/* 	printf("%i ",trees[split+i*(set_size+1)]); */
/*       printf("\n"); */
/*     } */

  }
  free(compat_matrix); free(trees_old);
  return(nsets);
}


/*= find all compat splits; low memory */
void up_trees(int numsp, int nspl, int cset_size, int *set, int *compat_matrix, 
	      int *ntrees, int *trees){
  int spl,j;

  for(spl = set[cset_size]; spl < nspl; spl++)
    if (compat_newsplit(nspl,cset_size,spl,set,compat_matrix)){
      for(j = 0; j < numsp-4; j++) 
	trees[j+(*ntrees)*(numsp-3)] = set[j];
      trees[numsp-4+(*ntrees)*(numsp-3)] = spl; 
      (*ntrees)++;
    }
}

void up_trees_coregp(int numsp, int nspl, int core_size, int cset_size, int *coregp, 
		     int *set_spl, int *set, int *compat_matrix, int *ntrees, int *trees){
  int spl,j;
  
  for(spl = set[cset_size]; spl < nspl; spl++)
    if (compat_newsplit(nspl,cset_size,spl,set,compat_matrix)){
      for(j = 0; j < core_size; j++) 
	trees[j+(*ntrees)*(numsp-3)] = coregp[j];
      for(j = core_size; j < numsp-4; j++)
	trees[j+(*ntrees)*(numsp-3)] = set_spl[set[j-core_size]];
      trees[numsp-4+(*ntrees)*(numsp-3)] = set_spl[spl]; 
      (*ntrees)++;
    }
}


int cset_comp(int nspl, int set_size, int *compat_matrix, int *set, int *cset){
  int spl,cset_size;

  for(spl = set[set_size-1]+1, cset_size=0; spl < nspl; spl++){
/*     printf("%i\n",spl); */
    if (compat_newsplit(nspl,set_size,spl,set,compat_matrix)) cset[cset_size++] = spl;
  }
  return(cset_size);
}

/* remove entries for core and splits incompatible with core */
int up_compat_matrix(int numsp, int nspl, int core_size, int last_spl, int *usplits, int *coregp, 
		      int *set_spl, int *compat_matrix){
  int i,j,nsplc;
  nsplc = cset_comp(nspl,core_size,compat_matrix,coregp,set_spl);
  for(i = 0; i < nsplc; i++)
    for(j = 0; j < nsplc; j++)
      compat_matrix[i+j*nsplc] = compat_pair(numsp,&usplits[set_spl[i]*numsp],
					    &usplits[set_spl[j]*numsp]);
  return(nsplc);
}

int get_nset(int nspl, int set_size, int el, int *set, int *compat_matrix, int *nset){
  int i,nset_size=0;
  for(i = 0; i < set_size; i++)
    if (compat_matrix[set[i]+el*nspl] == 1)
      nset[nset_size++] = set[i];
  return(nset_size);
}



int first_compat_split(int nspl, int cset_size, int *set, int *compat_matrix){
  int spl;

  for(spl = set[cset_size]; spl <= nspl; spl++){
/*     printf("%i %i %i\n", spl, set[cset_size-1], compat_newsplit(nspl,cset_size,spl,set,compat_matrix)); */
    if (compat_newsplit(nspl,cset_size,spl,set,compat_matrix)){
      set[cset_size] = spl;
      set[cset_size+1] = spl+1;
      return(1);
    }
  }
  return(0);    
}

void gnew_set(int nspl, int *set_size, int *cset_size, int *set, int *cset, int *compat_matrix){
  int i,j,nset_size,new_set=0;

  while (!new_set){
    if (*set_size == 1){
      set[0]++;
      *cset_size = cset_comp(nspl,*set_size,compat_matrix,set,cset);
      new_set = 1;
    }
    if (*set_size > 1){
      (*set_size)--;
      *cset_size = cset_comp(nspl,*set_size,compat_matrix,set,cset);
      for(i = 0; i < *cset_size; i++)
	if (cset[i] > set[(*set_size)]){
	  set[(*set_size)] = cset[i];
	  (*set_size)++;
	  for(j = i+1, nset_size = 0; j < *cset_size; j++) 
	    if (compat_matrix[set[(*set_size)-1]+cset[j]*nspl] == 1) cset[nset_size++] = cset[j];
	  *cset_size = nset_size;
	  new_set = 1;
	  break;
	}
    }
  }
    
}

int compat_splits_mem(int B, int numsp, int nspl, int maxtrees, int zeta, 
		      int *usplits, int *count, int *trees){
  int i,j,set_size,cset_size,nset_size,tr,proc;
  int *compat_matrix,*set,nsets,ntrees,ctr=0,core_size,*coregp,*set_spl,*nset,*cset;

  /* printf("nspl %i\n",nspl); */
  compat_matrix = (int *) malloc((size_t) nspl*nspl*sizeof(int));
  set = (int *) malloc((size_t) numsp*sizeof(int));
  coregp = (int *) malloc((size_t) nspl*sizeof(int));
  set_spl = (int *) malloc((size_t) nspl*sizeof(int));
  cset = (int *) malloc((size_t) nspl*sizeof(int));
  nset = (int *) malloc((size_t) nspl*sizeof(int));

  for(i = 0; i < nspl; i++)
    for(j = 0; j < nspl; j++)
      compat_matrix[i+j*nspl] = compat_pair(numsp,&usplits[i*numsp],
					    &usplits[j*numsp]);

  /* printf("    "); */
  /* for(i = 0; i < nspl; i++) printf("%3i ", i); printf("\n"); */
  /* for(i = 0; i < nspl; i++){ */
  /*   printf("%3i ",i); */
  /*   for(j = 0; j < nspl; j++) */
  /*     printf("%3i ", compat_matrix[i+j*nspl]); */
  /*   printf("\n"); */
  /* } */

  core_size = init_compat_splits(B,nspl,zeta,count,&nsets,coregp,compat_matrix);
  /* printf("core size: %i\n",core_size); */
  if (core_size == numsp-3){ /* core group is only tree */
    for(i = 0; i < core_size; i++) trees[i] = coregp[i];
    return(1);
  }
  if (core_size < 0) return(0); /* core group is not compatible */
  if (core_size == numsp-4){
    ntrees = 0;
    coregp[core_size] = coregp[core_size-1]+1;
    up_trees(numsp,nspl,core_size,coregp,compat_matrix,&ntrees,trees);
    return(ntrees);
  }
  
  /*  init_compat_splits for other compat_splits routine
   *  => core_size = 1 && nsets == 1 <=> core_size == 1
   *     core_size = 1 && nsets > 1 <=> core_size == 0 */
  if (core_size == 1 && nsets > 1){
    core_size = 0;
    for(i = 0; i < nspl; i++) set_spl[i] = i;
  }
  if (core_size >= 1 && nsets == 1){
    nspl = up_compat_matrix(numsp,nspl,core_size,coregp[core_size-1],usplits,coregp,
			    set_spl,compat_matrix);
  }

/*   printf("    "); */
/*   for(i = 0; i < nspl; i++) printf("%3i ", i); printf("\n"); */
/*   for(i = 0; i < nspl; i++){ */
/*     printf("%3i ",i); */
/*     for(j = 0; j < nspl; j++) */
/*       printf("%3i ", compat_matrix[i+j*nspl]); */
/*     printf("\n"); */
/*   } */
  
  ntrees = 0;
  ctr = 0;
  set_size = 1;
  set[0] = 0;
  cset_size = cset_comp(nspl, set_size, compat_matrix, set, cset);
/*   printf("core_size: %i\n", core_size); */

  while(core_size + nspl - set[0] + 1 >= numsp-3){

    proc = 0;
/*     printf("set:"); for(i = 0; i < set_size; i++) printf("%i ", set[i]); printf("\n"); */

    if (core_size + set_size == numsp-4){
      for(tr = 0; tr < cset_size; tr++){
	for(i = 0; i < core_size; i++)
	  trees[i+ntrees*(numsp-3)] = coregp[i];
	for(i = 0; i < set_size; i++)
	  trees[i+core_size+ntrees*(numsp-3)] = set_spl[set[i]];
	trees[numsp-4+ntrees*(numsp-3)] = set_spl[cset[tr]];
/* 	printf("ntrees: %i\n",ntrees+1); */
/* 	for(i = 0; i < numsp-3; i++) printf("%i ", trees[i+ntrees*(numsp-3)]); printf("\n"); */
	ntrees++;
	if (ntrees >= maxtrees) return(ntrees);
      }
      gnew_set(nspl,&set_size,&cset_size,set,cset,compat_matrix);
      proc = 1;
    }

    if (!proc && core_size + set_size + cset_size < numsp-3){
      gnew_set(nspl,&set_size,&cset_size,set,cset,compat_matrix);
      proc = 1;
    }

    if (!proc && core_size + set_size != numsp-4 && core_size + set_size + cset_size >= numsp-3){
      for(i = 1, nset_size = 0; i < cset_size; i++) 
	if (compat_matrix[cset[0]+cset[i]*nspl] == 1) nset[nset_size++] = cset[i];
      if (core_size + set_size + 1 + nset_size >= numsp-3){
	set[++set_size-1] = cset[0];
	cset_size = nset_size;
	for(i = 0; i < nset_size; i++) cset[i] = nset[i];
      }
      else{
	cset_size--;
	for(i = 0; i < cset_size; i++)  cset[i] = cset[i+1];
	if (cset_size == 0)
	  gnew_set(nspl,&set_size,&cset_size,set,cset,compat_matrix);
      }
    }

  }

  free(compat_matrix); free(set); free(coregp); free(set_spl); free(nset); free(cset);
  return(ntrees);
}

/*= convert sets of splits to trees */
int split_sum(int numsp, int *split, int *row_idx){
  int i,ssum;
  ssum = 0;
  for(i = 0; i < numsp; i++) 
    if (row_idx[i] >= 0) ssum += split[i];
  return(ssum);
}

void up_utree(int numsp, int join, int ssum, int *splits, int *row_idx, 
	      double *utreec){
  int pair_idx,r,l,i,np,pair[2];
  if (ssum == (numsp-join-2)) pair_idx = 0; /* get pair for join */
  if (ssum == 2) pair_idx = 1;
  np = 0; 
  for(i = 0; i < numsp; i++) 
    if (row_idx[i] >= 0 && splits[i] == pair_idx){
      pair[np] = row_idx[i];
      np++;
      if (np == 1) row_idx[i] = -1; /* update row */
      if (np == 2){
	row_idx[i] = join+numsp; /* update row */
	break;
      }
    }
  r = (pair[0] < pair[1])?pair[0]:pair[1];
  l = (pair[0] < pair[1])?pair[1]:pair[0];
  utreec[0] = r; utreec[1] = l; 
}

void final_joins(int numsp, int *row_idx, double *utreec){
  int isp,i,lab[3];

  utreec[5] = 2*numsp-3;
  isp = 0;
  for(i = 0; i < numsp; i++)
    if (row_idx[i] >= 0){
      lab[isp] = row_idx[i];
      isp++;
      if (isp > 2) break;
    }
  utreec[0] = (lab[0] < lab[1])?lab[0]:lab[1];
  utreec[1] = (lab[0] < lab[1])?lab[1]:lab[0];
  utreec[4] = lab[2];
}

void splits2tree(int B, int numsp, int nspl, int *tree, int *usplits, int *count, 
		 double *utreec){
  int i,j,r,ssum,join,split,*row_idx,*col_idx;
  double *bp;

  row_idx = (int *) malloc((size_t) numsp*sizeof(int));
  col_idx = (int *) malloc((size_t) nspl*sizeof(int));
  bp = (double *) malloc((size_t) (numsp-1)*sizeof(double));

  for(i = 0; i < numsp; i++) row_idx[i] = i;
  for(i = 0; i < nspl; i++) col_idx[i] = 1;
  for(i = 0; i < numsp-3; i++) col_idx[tree[i]] = 0;

  for(join = 0; join < numsp-3; join++){

    for(split = 0; split < nspl; split++){ /* find cherry */
      if (col_idx[split] == 0)
	ssum = split_sum(numsp,&usplits[split*numsp],row_idx);
      if (col_idx[split] == 0 && (ssum == 2 || ssum == (numsp-join-2))) break;
    }

    col_idx[split] = 1;
    up_utree(numsp,join,ssum,&usplits[split*numsp],row_idx,&utreec[join*4]);
    bp[join] = count[split]/((double) B);
  }

  final_joins(numsp, row_idx, &utreec[(numsp-3)*4]);

  for(i = 0; i < numsp-1; i++)
    for(j = 0; j < 2; j++){
      if ((r = (int) utreec[j+i*4]) >= numsp && r < 2*numsp-3){
	utreec[j+2+i*4] = bp[r-numsp];
      }
      else
	utreec[j+2+i*4] = 0.0;
    }

/*   printf("\n"); */
/*   pr_utreec(stdout,numsp,utreec); */
/*   printf("\n"); */
  free(row_idx); free(col_idx); free(bp);
}

/*= min bp for each of the trees */
void minbp_trees(int numsp, int ntrees, int *trees, int *count, int *minbpv){
  int tr;
  /* usplits sorted, max bp to min bp 
   * => min bp at largest indexed splits
   * tree split indeces sorted => min bp at last split */
  for(tr = 0; tr < ntrees; tr++){
    minbpv[tr] = count[trees[(numsp-4)+tr*(numsp-3)]];
  }
}

/*= find maxtree trees ranked according to minbp */
int ranked_compat_splits(int B, int numsp, int nspl, int maxtrees, int *usplits, int *count, 
			 int *trees){
  /* waste of computation, since trees are recomputed */
  /* there may be more trees with minbp >= last minbp */
  /* need storage for at least 2*ntrees trees */
  int zeta,spl,zetao,zetan,ntrees,i,j,tr,ntreest,nsplc,*treest;

  treest = (int *) malloc((size_t) maxtrees*(numsp-3)*sizeof(int));

  sort_splits(numsp,nspl,1,usplits,count);

/*   for(i = 0; i < nspl; i++){ */
/*     printf("%4i: ", i); */
/*     for(j = 0; j < numsp; j++) printf("%i ", usplits[j+i*numsp]); */
/*     printf("%4i\n", count[i]); */
/*   } */

  zeta = B;
  zetao = B+1;
  ntrees = 0;
  while(ntrees < maxtrees){
    for(spl = 0, nsplc = 0; spl < nspl; spl++)
      if (count[spl] < zeta){
	zetan = count[spl];
	break;
      }
      else
	nsplc++;
    if (nsplc >= numsp-3){
/*       printf("zeta: %i %i %i\n", zetao, zeta, zetan); */
      ntreest = compat_splits_mem(B,numsp,nsplc,maxtrees,zeta,usplits,count,treest);
/*       printf("ntreest: %i\n", ntreest); */
      for(tr = 0; tr < ntreest; tr++){
	for(i = 0; i < numsp-3; i++)
	  if (count[treest[i+tr*(numsp-3)]] < zetao){
	    for(j = 0; j < numsp-3; j++) 
	      trees[j+ntrees*(numsp-3)] = treest[j+tr*(numsp-3)];
/* 	    for(j = 0; j < numsp-3; j++) */
/* 	      printf("%i ", trees[j+ntrees*(numsp-3)]); printf("\n"); */
	    ntrees++;
	    if (ntrees >= maxtrees) return(ntrees);
	    break;
	  }
      }
    }
    if (spl >= nspl)
      return(ntrees);
    else{
      zetao = zeta;
      zeta = zetan;
    }
  }

  free(treest);
  return(ntrees);
}

/*= print trees in Newick format */
void pr_newick(FILE *outfile, int numsp, char names[][11], double *utreec){
  char tstr[10000];

  ut2n(numsp, utreec, names, tstr);
  fprintf(outfile,"%s\n",tstr);
}

void pr_names_R(FILE *outfile, int numsp, char name[][11]){
  int i,j,last_char;
  for(i = 0; i < numsp; i++){
    fprintf(outfile,"\"");
    last_char = 10;
    while(name[i][last_char] == '\0' || name[i][last_char] == ' ') last_char--;
    for(j = 0; j <= last_char; j++) fprintf(outfile,"%c",name[i][j]);
    fprintf(outfile,"\"\n");
  }
}

/*= get boostrap support for splits in a given tree */
int bp_split(int numsp, int nspl, int *split, int *usplits, int *count){
  int spl;
  for(spl = 0; spl < nspl; spl++){
    if (ivsame(numsp,split,&usplits[spl*numsp]))
      return(count[spl]);
  }
  return(-100); /* no split present */
}

void bp_tree(int B, int numsp,  int nspl, int *splits, int *usplits, int *count, 
	     double *utreec){
  /* possibly no support; not checking */
  int join,r,j;

  for(join = 0; join < numsp-1; join++)
    for(j = 0; j < 2; j++){
      r = (int) utreec[j+join*4];
      if (r >= numsp && r < 2*numsp-3)
	utreec[j+2+join*4] = bp_split(numsp, nspl, &splits[(r-numsp)*numsp],
				      usplits, count)/((double) B);
      else
	utreec[j+2+join*4] = 0.0;
    }
}

/*= splits in usplits that give tree */
int splits_in_tree(int numsp, int nspl, int *splits, int *usplits, int *tree){
  int spl,spl_tr,nsplt=0;

  for(spl = 0; spl < nspl; spl++)
    for(spl_tr = 0; spl_tr < numsp-3; spl_tr++)
      if (ivsame(numsp, &usplits[spl*numsp], &splits[spl_tr*numsp])){
	tree[nsplt++] = spl;
	break;
      }
  return(nsplt);
}
