#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "fn.h"
/*------------------------------------------------------------------------*/
int sp_char(char c){
  return((c == '(' || c == ')' || c == ',' || c == ':' || c == ';'));
}
/*------------------------------------------------------------------------*/
void ch_name(int lstring, char *tstring, int i){ 
  /* change name given by the first lstring chars of tstring to integer label i */
  char label[3];
  int tstl, llen;
  
  /* shorten the string and get rid of the name */
  tstl = 0;
  while(tstring[tstl+lstring] != '\0'){
    tstring[tstl] = tstring[tstl+lstring];
    tstl++;
  }
  tstring[tstl] = '\0';
  
  sprintf(label, "%i", i); /* i to char label */
  llen = strlen(label);   /* number of digits */
  
  /* make room for integer label; more efficient: combine this and shorten above */
  while(tstl >= 0){
    tstring[tstl+llen] = tstring[tstl];
    tstl--;
  }
  /* place the integer label where name used to be */
  for(tstl = 0; tstl < llen; tstl++)
    tstring[tstl] = label[tstl];
}
/*----------------------------------------------------------------------------*/
/* check whether there are branch lengths in tree */
int has_bl(char *tstring)
{
  while(*tstring != ';')
    if(*tstring == ':') /* signifies start of bl */
      return(1);
    else
      tstring++;
  return(0); /* must not be bl if we got this far */
}
/*----------------------------------------------------------------------------*/
void ch_labels_nbl(int ntaxa, char *tstring, char names[][11]){
  /* change all names in tstring to integer labels (no branch lengths)*/
  char taxname[11];
  int ltstring, i;

  while(*tstring != ';'){
    while(*tstring == ')' || *tstring == '(' || *tstring == ','){
      tstring++;
    }
    if(*tstring == ';') break;  /* end of tstring, no more names */

    ltstring = 0; /* beginning of new name */
    while(*tstring != ')' && *tstring != '(' && *tstring != ','){
      taxname[ltstring] = *tstring;
      tstring++;
      ltstring++;
    }

    for(i=ltstring; i < 10; i++) taxname[i] = ' '; /* pad w/ blanks */
    taxname[10] = '\0'; /* name is in taxname */
    for(i = 0; i < ltstring; i++) tstring--; /* go to start of name */
    
    for(i = 0; i < ntaxa; i++){            /* check which taxa it is */
      if (strncmp(taxname, names[i], 10) == 0){ /* i is label for taxa */
	ch_name(ltstring, tstring, i);	  
	break;
      }
    }
    /* go to end of name */
    while(*tstring != '(' && *tstring != ')' && *tstring != ',') tstring++;
  }
}
/*------------------------------------------------------------------------*/
void ch_labels(int ntaxa, char *tstring, char names[][11]){
  /* change all names in tstring to integer labels */
  char taxname[11];
  int lstring, i;
  
  while(*tstring != ';'){
    while(*tstring != ':' && *tstring != ';'){ /* name always before : */
      tstring++;
    }
    if (*tstring == ';') break;  /* end of tstring, no more names */
    tstring--; lstring = 0;      /* end of a possible name */
    while(!sp_char(*tstring)){   /* find the beginning */
      tstring--; lstring++;
    }
    if (*tstring == '(' || *tstring == ','){ /* condition for a name */
      tstring++;
      strncpy(taxname, tstring, lstring);    /* name is in taxname */
      for(i=lstring; i < 10; i++) taxname[i] = ' '; /* pad w/ blanks */
      taxname[10] = '\0';

/*       for(i = 0; i < lstring; i++) putchar(taxname[i]); printf("\n");  */
      for(i = 0; i < ntaxa; i++){            /* check which taxa it is */
/*  	printf("%s\n", names[i]);  */
	if (strncmp(taxname, names[i], 10) == 0){ /* i is label for taxa */
	  ch_name(lstring, tstring, i);	  
	  break;
	}
      }
/*       printf("%s\n\n", tstring); */ 
    }
    while(*tstring != ':') tstring++; /* get back to : */
    tstring++; /* process the rest of string */
  }
}
/*------------------------------------------------------------------------*/
void gettl(char *pair, int *r, int *l, double *lr, double *ll){
  /* get the labels r < l of the branches to be joined and their lengths lr, ll*/
  char taxa[4], length[80];
  int i, bl = 1;
  double tmp;
  
  /* form of pair on input is label1:length,label2:length 
     or possibly label1,label2.. (no l branch length)
  */
  i = 0;
  while(*pair != ':' && *pair != ','){ /* find first label1 */
    taxa[i++] = *pair;
    pair++;
  }
  if (*pair == ','){/* no branch length */
    *ll = 0.0; 
    bl = 0;
  }
  taxa[i] = '\0';
  *l = atoi(taxa);  /* change to integer */

  /* form of pair is now :length,label2:length 
     or ,label2... (no r branch length) */
  if (bl){
    pair++; 
    i = 0;
    while(*pair != ','){ /* find first length */
      length[i++] = *pair;
      pair++;
    }
    length[i]='\0';
    *ll = atof(length); /* change to double */
  }
  
  /* form of pair is now ,label2:length 
     or ,label2 (no r branch lengths) */
  pair++;
  i = 0;
  while(*pair != ':' && *pair != '\0'){ /* find second label */
    taxa[i++] = *pair;
    pair++;
  }
  if (*pair == '\0'){/* no branch length */
    *lr = 0.0;
    bl = 0;
  }
  else
    bl = 1;
  taxa[i] = '\0';
  *r = atoi(taxa);
  
   /* form of pair is now :length */
  if (bl){
    pair++;
    *lr = atof(pair);
  }
    
  if (*r > *l){ /* always arranged so that r < l on return */
    i = *r; *r = *l; *l = i;
    tmp = *lr; *lr = *ll; *ll = tmp;
  }
}
/*------------------------------------------------------------------------*/
int isapair(char *tstring, int *l, double *utreec){
  /* tstring is currently "(...", return 1 if start is a pair: (l1:len,l2,len)
   * l = max(l1,l2) is used later to adust the labels of the branches in tstring
   * tstring is modified so that the pair (l1:len,l2,len) is replaced by 
   * r = min(l1,l2)
   */
  int lpair, tstl, r, lrc,i, multiple;
  double ll, lr;
  char rc[4], pair[1000];

  /* pair = l1:len,l2:len if this is a pair */
  lpair = 0;
  while(tstring[lpair+1] != ')' && tstring[lpair+1] != '(' && 
	tstring[lpair+1] != ',' ){ /* first half of pair if it is one */
    pair[lpair] = tstring[lpair+1];
    lpair++;
  }
  if (tstring[lpair+1] == '(') return(0); /* not a pair */
  pair[lpair] = tstring[lpair+1];
  lpair++;
  while(tstring[lpair+1] != ')' && tstring[lpair+1] != '(' && 
	tstring[lpair+1] != ',' ){ /* second half of pair if it is one */
    pair[lpair] = tstring[lpair+1];
    lpair++;
  }
  if (tstring[lpair+1] == '(') return(0); /* not a pair */
  if (tstring[lpair+1] == ',') 
    multiple = 1;
  else
    multiple = 0;

  /* pair of the form (l1:len,l2:len) or (l1:len,l2:len, */
  pair[lpair]='\0';
   /* printf("%s\n", pair); */

  /* remove (l1:len,l2:len) or (l1:len,l2:len, from tstring */
  tstl = lpair+2;
  while(tstring[tstl] != '\0'){
    tstring[tstl-(lpair+2)] = tstring[tstl];
    tstl++;
  }
  tstring[tstl-(lpair+2)] = '\0';
  
  gettl(pair,&r,l,&lr,&ll); /* r = min(l1,l2) ... */
  *utreec = r; utreec++;
  *utreec = *l; utreec++;
  *utreec = lr; utreec++;
  *utreec = ll; utreec++; /* row of a treec file */

  sprintf(rc, "%i", r); /* change r to char */
  lrc = strlen(rc); 

  /* place r where (l1:len,l2,len) was; efficiency: combine with remove above */
  if (multiple) lrc++;
  tstl = tstl-(lpair+2); 
  for(i = tstl; i >= 0; i--){ /* make room */
    tstring[i+lrc] = tstring[i];
  }
  if (multiple){
    tstring[0] = '(';
    for(i = 1; i < lrc; i++) tstring[i] = rc[i-1];
  }
  else{
    for(i = 0; i < lrc; i++)  /* insert r */
      tstring[i] = rc[i];
  }
  tstl = tstl+lrc;

  /* if pair = (l1:len,l2:len, need to insert :0.0, */
  if (multiple){
    for(i = tstl; i >= 0; i--){ /* make room */
      tstring[i+5] = tstring[i];
    }
    tstring[lrc] = ':';
    tstring[lrc+1] = '0';
    tstring[lrc+2] = '.';
    tstring[lrc+3] = '0';
    tstring[lrc+4] = ',';
  }
  return(1);
} 
/*------------------------------------------------------------------------*/
int pairtreec(char *tstring, double *utreec){ 
  /* tstring is modified so that the first pair (l1:len,l2,len) found is replaced 
   * by r = min(l1,l2)
   */
  int l;
  while(*tstring != '\0'){
    while(*tstring != '(') tstring++; /* start of a possible pair */
    if (isapair(tstring, &l, utreec)){  /* l = max(l1,l2) */
      break;
    }
    else{
      tstring++;  /* keep checking */
    }
  }
  return(l);
}
/*------------------------------------------------------------------------*/
void less_dig(char *tstring){
  /* special code to reduce the length of tstring when a label changes from
   * 2 digit to 1 digit or 3 to 2
   * has not been tested
   */
  while(*(tstring+1) != '\0'){
    *tstring = *(tstring+1);
    tstring++;
  }
  *tstring = '\0';
}
/*------------------------------------------------------------------------*/
void update_tlab(char *tstring, int l){
  /* modifies tstring
   * the larger branch in the pair being joined has label l. adjust labels larger 
   * than l by subtracting 1 from them
   */
  int i, itaxa;
  char taxa[4];
  while(*tstring != '\0'){
    /* start of label should be first non '(', ')', ',', ';' */
    while (*tstring == '(' || *tstring == ')' || *tstring == ',' || 
	   *tstring == ';')
      tstring++;
    if (*tstring == '\0') break;
    
    i = 0;
    while(*(tstring+i) != ':' && *(tstring+i) != ',' && *(tstring+i) != '(' &&
	  *(tstring+i) != ')'){/* copy label to taxa */
      taxa[i] = *(tstring+i);
      i++;
    }
    taxa[i] = '\0';
    itaxa = atoi(taxa); /* integer representation */

    if (itaxa > l){ /* label = label - 1 */
      itaxa--;      /* integer representation */
      sprintf(taxa, "%i", itaxa); /* char represntation */
      i = 0;
      while(taxa[i] != '\0'){  /* replace digits by label - 1 */
	*tstring = taxa[i];
	tstring++; i++;
      }  /* tstring was label:... now :... */
      /* will need one less digit if itaxa was 100 or 10 */
      if (itaxa == 9 || itaxa == 99)
	less_dig(tstring);
    }
    while(*tstring != '(' && *tstring != ')' && *tstring != ',')
      tstring++; /* move past branch length if present */
  }
}
/*------------------------------------------------------------------------*/
void utriple(char *tstring, double *utreec){
  /* unrooted case => tstring (l1:len,l2:len,l3:len) at end
   * print out l1, l2, len len and
   *           0 1 0.0 len artificial root
   */
  /* probably not needed because of changes to isapair to handle special case */
  int i, r, l;
  double ll, lr, tmp;
  char length[100], taxa[4];
  
  i = 0;
  tstring++;
  while(*tstring != ':'){/* find first label1 */
    taxa[i++] = *tstring;
    tstring++;
  }
  taxa[i] = '\0';
  l = atoi(taxa);  /* change to integer */

  /* form of tstring is now :len,l2:len,l3:len); */
  tstring++; 
  i = 0;
  while(*tstring != ','){ /* find first length */
    length[i++] = *tstring;
    tstring++;
  }
  length[i] = '\0';
  ll = atof(length); /* change to double */
  
  /* form of tstring is now ,l2:len,l3:len); */
  tstring++;
  i = 0;
  while(*tstring != ':'){ /* find second label */
    taxa[i++] = *tstring;
    tstring++;
  }
  taxa[i] = '\0';
  r = atoi(taxa);
  
   /* form of tstring is now :len,l3:len); */
  tstring++;
  i = 0;
  while(*tstring != ','){
    length[i++] = *tstring;
    tstring++;
  }
  length[i] = '\0';
  lr = atof(length); /* change to double */
  if (r > l){ /* always arranged so that r < l on return */
    i = r; r = l; l = i;
    tmp = lr; lr = ll; ll = tmp;
  }
  *utreec = r; utreec++;
  *utreec = l; utreec++;
  *utreec = lr; utreec++;
  *utreec = ll; utreec++;
  
  /* form of tstring is now :len,l3:len); */
  while(*tstring != ':') tstring++;
  tstring++; /* tstring is len); */
  i = 0;
  while(*tstring != ')'){
    length[i++] = *tstring;
    tstring++;
  }
  length[i] = '\0';
  ll = atof(length); /* change to double */
  *utreec = 0.0; utreec++;
  *utreec = 1.0; utreec++;
  *utreec = 0.0; utreec++;
  *utreec = ll; utreec++;
}
/*------------------------------------------------------------------------*/
void treecsu(int ntaxa, char *tstring, double *utreec){
  /* tstring constains the PHYLIP format tree (..)
   * unrooted case: requires special handling of the triple at the end  */
  /* probably not needed because of changes to isapair to handle special case */
  int i, l, j;
  for(i = 0; i < ntaxa - 3; i++){ /* always ntaxa - 1 joins to get the tree */
    l = pairtreec(tstring, utreec); /* lpair of branches is joined */
    for(j = 0; j < 4; j++) utreec++;
    update_tlab(tstring, l); /* labels of remaining branches adjusted */
    /*  printf("%s\n", tstring); */
  }
  utriple(tstring, utreec); /* tstring now (l1:len,l2:len,l3:len) */
}
/*------------------------------------------------------------------------*/
void treecs(int ntaxa, char *tstring, double *utreec){
  /* tstring constains the PHYLIP format tree (..); 
   * output to utreec (4 x (numsp-1)) the transpose of utreec file 
   */
  int i, l, j;
/*   printf("%s\n",tstring); */
  for(i = 0; i < ntaxa - 1; i++){ /* always ntaxa - 1 joins to get the tree */
    l = pairtreec(tstring, utreec); /* lpair of branches is joined */
    for(j = 0; j < 4; j++) {
       /* printf("%f ", *utreec); */
      utreec++;
    }
     /* printf("\n"); */
     if (i < ntaxa - 2) /* labels of remaining branches adjusted */
       update_tlab(tstring, l); 
    /* printf("%s\n", tstring); */
  }
}
/*------------------------------------------------------------------------*/
int read_tree(FILE *fp, char *tstring)
{
  int ltstring;
  char c;

  ltstring = 0;
  while((c = getc(fp)) != ';'){   /* get rid of white space */
    if (c != ' ' && c != '\t' && c != '\r' && c != '\n')
      tstring[ltstring++]=c;
  } 
  tstring[ltstring++]=';';
  tstring[ltstring++]='\0';
  return(ltstring);
}
/*------------------------------------------------------------------------*/
double *read_utreec(FILE *fp, int ntaxa, char name[][11], int new){
  char tstring[10000];
  double *outreec, *utreec;
  
  read_tree(fp,tstring);
  fclose(fp); 
  if (has_bl(tstring))
    ch_labels(ntaxa,tstring,name);  
  else
    ch_labels_nbl(ntaxa,tstring,name); 
  
  outreec = (double *) malloc((size_t) (ntaxa-1)*4*sizeof(double));
  treecs(ntaxa,tstring,outreec);
  if(new){
    utreec = (double *) malloc((size_t) (ntaxa-1)*4*sizeof(double));
    ou2nu(ntaxa,outreec,utreec);
    free(outreec);
    return(utreec);
  }
  else{
    return(outreec);
  }
}
/*------------------------------------------------------------------------*/
double *read_utreecm(FILE *fp, int ntree, int ntaxa, char name[][11]){
  /* returns nutreec */
  char tstring[10000];
  int itree;
  double *outreec, *utreec;
  
  utreec = (double *) malloc((size_t) ntree*(ntaxa-1)*4*sizeof(double));
  outreec = (double *) malloc((size_t) (ntaxa-1)*4*sizeof(double));
  for(itree = 0; itree < ntree; itree++){
    read_tree(fp,tstring);
    if (has_bl(tstring))
      ch_labels(ntaxa,tstring,name);  
    else
      ch_labels_nbl(ntaxa,tstring,name); 
    treecs(ntaxa,tstring,outreec);
/*     pr_utreec(stdout,ntaxa,outreec); printf("\n"); */
    ou2nu(ntaxa,outreec,&utreec[itree*(ntaxa-1)*4]);
/*     pr_utreec(stdout,ntaxa,&utreec[itree*(ntaxa-1)*4]); printf("\n\n"); */
/*     if (itree == 3) exit(0); */
  }
  fclose(fp); 
  free(outreec);
  return(utreec);
}
